/*****************************************************************
UZI (Unix Z80 Implementation) Sharp MZ-800 video init:  mzvidini.c
******************************************************************/

#include "font.h"

/* Screen constants */
/* The constants must match in boot and main module */
#define SCRN_MAX_X 80
#define SCRN_MAX_Y 25
#define SCRN_FONT_ADDR 0c000h - 8 * 256			/* ISO font is stored at the end of the second screen */
#define SCRN_LINE_ADDR_POS SCRN_FONT_ADDR - 25 * 2;	/* Screen line addresses are stored just before it */

/**
 * Initializes video hardware
 */
void mz_init_video() {

#asm
	di
	ld	a, 4		; we're using 640x200 in 2 colors video mode
	out	(0ceh), a	; later, second plane can be used for hosting some code
				; to get MZI load above 0x8000 (current start is at 7600h)
				
	ld	a, 0		; set pallete
	out	(0f0h), a
	ld	a, 1fh
	out	(0f0h), a

	ld	a, 8fh		; clear video ram
	out	(0cch), a
	out	(0cdh), a
	ld	hl, 8000h
	ld	de, 8001h
	ld	bc, 3fffh
	in	a, (0e0h)
	ld	(hl), l
	ldir
	
	ld	a, 4
	out	(0cch), a	; put the line addresses in the video memory
	ld	hl, 8000h
	ld	de, SCRN_LINE_ADDR_POS
	ld	bc, 80 * 8
	ld	a, 25
loop1:	ex	de, hl
	ld	(hl), e
	inc	hl
	ld	(hl), d
	inc	hl
	ex	de, hl
	add	hl, bc
	dec	a
	jr	nz, loop1
	
	ld	hl, _SCRN_FONT	; put the font data in the video memory
	ld	bc, 256 * 8
	ldir
	in	a, (0e1h)

	ld	a, 1		; set working video access mode
	out	(0cch), a	; this can be used for whole MZI session without further changes
	ld	a, 4		; we're accessing just one plane, the second one is left free for other stuff (font,..)
	out	(0cdh), a	; we'll be just reading from it
#endasm
  SCRN_FONT;	/* this is here just for HTC to not complain */
}
