#include <stdio.h>
#include <stdlib.h>

#define BUFFER_SIZE 65536
#define LINE_LENGTH 16


unsigned char mirror_char( unsigned char c ) {
  unsigned char out = 0;
  int i = 0;
  while( i++ < 8 ) {
    out <<= 1;
    out |= c & 1;
    c >>= 1;
  }
  return out;
}


int main( int argc, char **argv ) {
  FILE *rfile;
  int i, size, header = 1, line_count = 0;
  unsigned char read_buffer[ BUFFER_SIZE ];

  if( argc < 3 ) {
    fprintf( stderr, "Converts raw font data into C character structure with mirrored bytes (for use in Sharp MZ-800 VRAM)\n" );
    fprintf( stderr, " Syntax: %s <source_font_file> <variable_name>\n", argv[ 0 ] );
    return( 0 );
  }
  
  if( ( rfile = fopen( argv[ 1 ], "r" ) ) == NULL ) {
    fprintf( stderr, "Error reading the file.\n" );
    return( 0 );
  }
  
  while( ! feof( rfile ) ) {
    size = fread( &read_buffer, 1, BUFFER_SIZE, rfile );
    if( size > 0 ) {
      if( header ) {
        printf( "static unsigned char %s[ %i ] = {\n", argv[ 2 ], size ); // a quick hack - size should not be one-read dependent
        header = 0;
      }
      for( i = 0; i < size; i++ ) {
        printf( " 0x%02x", mirror_char( read_buffer[ i ] ) );

        if( i + 1 < size || ! feof( rfile ) )
          printf( "," );

        if( ++line_count == LINE_LENGTH ) {
          printf( "\n" );
          line_count = 0;
        }
      }
    }
    if( feof( rfile ) && ! header ) {
      printf( "};\n" );
    }
  }
  
  fclose( rfile );
}
