#include <stdio.h>
#include <stdlib.h>

#define BUFFER_SIZE 65536


/**
 * Converts the string to the number in a very simple and buggy way.
 * Atoi itself does not support hexa values and sscanf has broken memory management ;-(
 */
int atoi_local( char *in ) {
  int result = 0;
  int len = strlen( in );
  int multiplier = 1;
  int base = 10;
  int i, value;
  char c;

  for( i = 0; i < len; i++ ) {
    c = in[ i ];
    value =
        c >= 'A' && c <= 'F' ? c - 'A' + 10 :
        c >= 'a' && c <= 'f' ? c - 'a' + 10 :
        c >= '0' && c <= '9' ? c - '0' : -1;
    if( c == 'x' || c == 'X' )
      base = 16;
    if( value != -1 )
      result = result * base + value;
  }
  return result;
}


int main( int argc, char **argv ) {
  FILE *rfile;
  FILE *wfile;
  unsigned int i, namesize, size, begin, exec, beginm, execm, pos = 0;
  unsigned char read_buffer[ BUFFER_SIZE ];
  unsigned char head_buffer[ 128 ];

  if( argc < 8 ) {
    fprintf( stderr, "Makes MZF file from a binary file\n" );
    fprintf( stderr, " Syntax: %s <source_file> <destination_file> <MZ_name> <MZ_begin> <MZ_exec> <MZ_begin_main> <MZ_exec_main>\n" );
    return( 0 );
  }
  
  if( ( rfile = fopen( argv[ 1 ], "r" ) ) == NULL ) {
    fprintf( stderr, "Error reading the file.\n" );
    return( 0 );
  }
  if( ( wfile = fopen( argv[ 2 ], "w+" ) ) == NULL ) {
    fprintf( stderr, "Error writing the file.\n" );
    return( 0 );
  }

  // get the data
  size = fread( &read_buffer, 1, BUFFER_SIZE, rfile );

  // make the header
  head_buffer[ pos++ ] = 1;
  namesize = strlen( argv[ 3 ] );
  for( i = 0; i < 16; i++ )
    head_buffer[ pos++ ] = i < namesize ? argv[ 3 ][ i ] : i == namesize ? 0xd : 0;
  head_buffer[ pos++ ] = namesize >= 16 ? 0xd : 0;
  begin = atoi_local( argv[ 4 ] );
  exec = atoi_local( argv[ 5 ] );
  beginm = atoi_local( argv[ 6 ] ) + begin;
  execm = atoi_local( argv[ 7 ] );
  head_buffer[ pos++ ] = size & 0xff;
  head_buffer[ pos++ ] = ( size >> 8 ) & 0xff;
  head_buffer[ pos++ ] = begin & 0xff;
  head_buffer[ pos++ ] = ( begin >> 8 ) & 0xff;
  head_buffer[ pos++ ] = exec & 0xff;
  head_buffer[ pos++ ] = ( exec >> 8 ) & 0xff;
  head_buffer[ pos++ ] = beginm & 0xff;
  head_buffer[ pos++ ] = ( beginm >> 8 ) & 0xff;
  head_buffer[ pos++ ] = execm & 0xff;
  head_buffer[ pos++ ] = ( execm >> 8 ) & 0xff;
  
  // write the data
  fwrite( &head_buffer, 1, 128, wfile );
  if( size > 0 )
    fwrite( &read_buffer, 1, size, wfile );
  
  fclose( wfile );
  fclose( rfile );
}
